package net.sf.seaf.persistence.exception;

/**
 * Thrown when {@link net.sf.seaf.persistence.util.CodeTablePropertyRefresher}
 * cannot refresh a code table property.
 */
public class CodeTablePropertyRefreshingException extends
		SeafPersistenceException {

	private final String propertyName, code;
	private final Class<?> entityClass, propertyType;

	/**
	 * Full constructor.
	 * 
	 * @param entityClass
	 *            Class of the entity being refreshed
	 * @param propertyType
	 *            Type of the property that cannot be refreshed
	 * @param propertyName
	 *            Name of the property that cannot be refreshed
	 * @param code
	 *            Code by which the property is refreshed
	 * @param cause
	 *            Cause exception
	 */
	public CodeTablePropertyRefreshingException(Class<?> entityClass,
			Class<?> propertyType, String propertyName, String code,
			Throwable cause) {
		super("Could not refresh property "
				+ (null == entityClass ? null : entityClass.getName()) + "."
				+ propertyName + " as "
				+ (null == propertyType ? null : propertyType.getSimpleName())
				+ " with code [" + code + "]", cause);
		this.entityClass = entityClass;
		this.propertyType = propertyType;
		this.propertyName = propertyName;
		this.code = code;
	}

	/**
	 * Returns class of the entity being refreshed.
	 * 
	 * @return Class of the entity being refreshed
	 */
	public Class<?> getEntityClass() {
		return entityClass;
	}

	/**
	 * Returns type of the property that cannot be refreshed.
	 * 
	 * @return Type of the property that cannot be refreshed
	 */
	public Class<?> getPropertyType() {
		return propertyType;
	}

	/**
	 * Returns name of the property that cannot be refreshed.
	 * 
	 * @return Name of the property that cannot be refreshed
	 */
	public String getPropertyName() {
		return propertyName;
	}

	/**
	 * Returns code by which the property is refreshed.
	 * 
	 * @return Code by which the property is refreshed
	 */
	public String getCode() {
		return code;
	}

	private static final long serialVersionUID = 1L;

}
