package net.sf.seaf.persistence.hibernate;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.List;

import net.sf.seaf.persistence.CriteriaBean;
import net.sf.seaf.persistence.GenericDAO;

import org.hibernate.Criteria;
import org.hibernate.ScrollableResults;
import org.hibernate.Session;
import org.hibernate.criterion.DetachedCriteria;
import org.springframework.orm.hibernate3.HibernateCallback;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

/**
 * Generic Hibernate DAO.
 * 
 * @param <E>
 *            The entity
 * @param <ID>
 *            The primary key class
 * @param <EC>
 *            The bean of entity criteria
 * @param <P>
 *            Property enum
 */
public abstract class GenericHibernateDAO<E, ID extends Serializable, EC extends CriteriaBean<P>, P extends Enum<?>>
		extends HibernateDaoSupport implements GenericDAO<E, ID, EC, P> {

	private final Class<E> persistentClass;

	@SuppressWarnings("unchecked")
	public GenericHibernateDAO() {
		persistentClass = (Class<E>) ((ParameterizedType) getClass()
				.getGenericSuperclass()).getActualTypeArguments()[0];
	}

	/**
	 * Returns the class of the entity.
	 * 
	 * @return Class<E> The class of the entity
	 */
	public Class<E> getPersistentClass() {
		return persistentClass;
	}

	private HibernateCriteriaBuilder criteriaBuilder;

	/**
	 * Returns the current HibernateCriteriaBuilder.
	 * 
	 * @return HibernateCriteriaBuilder
	 */
	public HibernateCriteriaBuilder getCriteriaBuilder() {
		return criteriaBuilder;
	}

	/**
	 * Sets the HibernateCriteriaBuilder to be used.
	 * 
	 * @param criteriaBuilder
	 *            New HibernateCriteriaBuilder
	 */
	public void setCriteriaBuilder(HibernateCriteriaBuilder criteriaBuilder) {
		this.criteriaBuilder = criteriaBuilder;
	}

	// business methods:

	public E create(E entity) {
		getHibernateTemplate().save(entity);
		return entity;
	}

	@SuppressWarnings("unchecked")
	public E retrieve(ID id) {
		return (E) getHibernateTemplate().load(getPersistentClass(), id);
	}

	@SuppressWarnings("unchecked")
	public E retrieveIfExists(ID id) {
		return (E) getHibernateTemplate().get(getPersistentClass(), id);
	}

	public E update(E entity) {
		getHibernateTemplate().update(entity);
		return entity;
	}

	@SuppressWarnings("unchecked")
	public E merge(E entity) {
		return (E) getHibernateTemplate().merge(entity);
	}

	public void delete(E entity) {
		getHibernateTemplate().delete(entity);
	}

	public E createOrUpdate(E entity) {
		getHibernateTemplate().saveOrUpdate(entity);
		return entity;
	}

	@SuppressWarnings("unchecked")
	public List<E> find(EC criteria) {
		DetachedCriteria hc = prepareHibernateCriteria(criteria);
		return getHibernateTemplate().findByCriteria(hc);
	}

	@SuppressWarnings("unchecked")
	public List<E> find(EC criteria, int fromRecNo, int count) {
		DetachedCriteria hc = prepareHibernateCriteria(criteria);
		return getHibernateTemplate().findByCriteria(hc, fromRecNo - 1, count);
	};

	public int size(EC criteria) {
		final DetachedCriteria hc = prepareHibernateCriteria(criteria);
		return (Integer) getHibernateTemplate().execute(
				new HibernateCallback() {
					public Integer doInHibernate(Session session) {
						Criteria executableCriteria = hc.getExecutableCriteria(session);
						ScrollableResults cursor = executableCriteria.scroll();
						cursor.last();
						return 1 + cursor.getRowNumber();
					}
				}, true);
	};

	private DetachedCriteria prepareHibernateCriteria(EC criteria) {
		DetachedCriteria hc = criteriaFactory.getInstance(getPersistentClass());
		getCriteriaBuilder().addCriteria(hc, criteria);
		return hc;
	}

	public void flush() {
		getHibernateTemplate().flush();
	}

	// criteria instantiation:

	public interface CriteriaFactory {
		DetachedCriteria getInstance(Class<?> persistentClass);
	}

	private CriteriaFactory criteriaFactory = new CriteriaFactory() {
		public DetachedCriteria getInstance(Class<?> persistentClass) {
			return DetachedCriteria.forClass(persistentClass);
		}
	};

	/**
	 * Set factory for criteria.
	 * 
	 * @param criteriaFactory
	 */
	public void setCriteriaFactory(CriteriaFactory criteriaFactory) {
		this.criteriaFactory = criteriaFactory;
	}

}
