package net.sf.seaf.persistence.hibernate;

import static org.hibernate.criterion.Restrictions.eq;
import static org.hibernate.criterion.Restrictions.ge;
import static org.hibernate.criterion.Restrictions.gt;
import static org.hibernate.criterion.Restrictions.isNotNull;
import static org.hibernate.criterion.Restrictions.isNull;
import static org.hibernate.criterion.Restrictions.le;
import static org.hibernate.criterion.Restrictions.like;
import static org.hibernate.criterion.Restrictions.lt;
import static org.hibernate.criterion.Restrictions.ne;
import static org.hibernate.criterion.Restrictions.or;
import net.sf.seaf.common.Operator;
import net.sf.seaf.common.PropertySort;
import net.sf.seaf.persistence.CriteriaBean;
import net.sf.seaf.persistence.MasterPkCriterion;
import net.sf.seaf.persistence.exception.SeafPersistenceException;
import net.sf.seaf.util.bean.PropertyValueExtractor;

import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.DetachedCriteria;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Builds Hibernate criteria from entity criteria.
 */
public class HibernateCriteriaBuilder {

	Logger log = LoggerFactory.getLogger(HibernateCriteriaBuilder.class);

	/**
	 * Adds the entity criteria to the Hibernate criteria, including the
	 * operators and sort criteria.
	 * 
	 * @param hibernateCriteria
	 *            The Hibernate criteria
	 * @param criteriaBean
	 *            The entity criteria
	 */
	public void addCriteria(DetachedCriteria hibernateCriteria,
			CriteriaBean<?> criteriaBean) {
		log.info("Setting criteria for bean {}", criteriaBean.getClass()
				.getName());
		PropertyValueExtractor extractor = new PropertyValueExtractor(
				criteriaBean);
		addOwnCriteria(hibernateCriteria, criteriaBean, extractor);
		addMasterPkCriteria(hibernateCriteria, criteriaBean, extractor);
		addSort(hibernateCriteria, criteriaBean);
	}

	/**
	 * Add search criteria on own properties.
	 * 
	 * @param hibernateCriteria
	 * @param criteriaBean
	 * @param extractor
	 */
	private void addOwnCriteria(DetachedCriteria hibernateCriteria,
			CriteriaBean<?> criteriaBean, PropertyValueExtractor extractor) {
		for (Object propertyName : criteriaBean.getPropertyNames()) {
			String name = propertyName.toString();
			Object value = extractor.extractPropertyValue(name);
			Operator op = extractOperatorValue(extractor, name);
			if (null != op || (null != value && !(Boolean.FALSE.equals(value)))) {
				if (log.isDebugEnabled()) {
					log.debug("Setting criteria {} {} {}", new Object[] { name,
							(null == op) ? "[default]" : op.getSql(), value });
				}
				hibernateCriteria.add(getCriterion(name, value, op));
			} else {
				log.trace("Criteria on {} not set, operator is null "
						+ "and value is null or false: {}", name, value);
			}
		}
	}

	/**
	 * Add search criteria on master entity primary key properties.
	 * 
	 * @param hibernateCriteria
	 * @param criteriaBean
	 * @param extractor
	 */
	private void addMasterPkCriteria(DetachedCriteria hibernateCriteria,
			CriteriaBean<?> criteriaBean, PropertyValueExtractor extractor) {
		for (MasterPkCriterion criterion : criteriaBean.getMasterPkCriteria()) {
			String name = criterion.getCriteriaProperty();
			Object value = extractor.extractPropertyValue(name);
			String associationPath = name.substring(0, name.length() - 2);
			if (null != value) {
				log.debug("Setting criteria "
						+ "on master entity primary key {}={}", name, value);
				hibernateCriteria.createCriteria(associationPath).add(
						eq(criterion.getMasterPkProperty(), value));
			} else {
				log.trace("Master criteria on {} not set, value is null", name);
			}
		}
	}

	/**
	 * Add sort criteria.
	 * 
	 * @param hibernateCriteria
	 * @param criteriaBean
	 */
	private void addSort(DetachedCriteria hibernateCriteria,
			CriteriaBean<?> criteriaBean) {
		for (PropertySort<?> propertySort : criteriaBean.getSortCriteria_()) {
			log.debug("Adding sort {} {}", propertySort.getProperty(),
					propertySort.getOrder());
			hibernateCriteria.addOrder(new OrderAdapter(propertySort));
		}
	}

	/**
	 * Extracts the value of an operator for a property.
	 * 
	 * @param extractor
	 *            Extractor
	 * @param propertyName
	 *            Name of the property
	 * @return Operator The operator of the property
	 */
	private Operator extractOperatorValue(PropertyValueExtractor extractor,
			String propertyName) {
		return (Operator) extractor.extractPropertyValue(propertyName + "_Op");
	}

	/**
	 * Returns the Hibernate Criterion for a property.
	 * <p>
	 * When the operator is not specified (is null), then it defaults to
	 * <code>like</code> for String value or <code>equals</code> otherwise.
	 * 
	 * @param propertyName
	 *            The name of the property
	 * @param propertyValue
	 *            The value of the property
	 * @param op
	 *            The operator of the property
	 * @return Criterion The Hibernate Criterion
	 */
	private Criterion getCriterion(String propertyName, Object propertyValue,
			Operator op) {
		if (Operator.like == op
				|| (null == op && propertyValue instanceof String))
			return like(propertyName, propertyValue);
		if (null == op || Operator.eq == op)
			return eq(propertyName, propertyValue);
		if (Operator.gt == op)
			return gt(propertyName, propertyValue);
		if (Operator.ge == op)
			return ge(propertyName, propertyValue);
		if (Operator.lt == op)
			return lt(propertyName, propertyValue);
		if (Operator.le == op)
			return le(propertyName, propertyValue);
		if (Operator.neq == op)
			return ne(propertyName, propertyValue);
		if (Operator.ex == op)
			return isNotNull(propertyName);
		if (Operator.nex == op)
			return isNull(propertyName);
		if (Operator.gtex == op)
			return or(gt(propertyName, propertyValue), isNull(propertyName));
		if (Operator.geex == op)
			return or(ge(propertyName, propertyValue), isNull(propertyName));
		if (Operator.ltex == op)
			return or(lt(propertyName, propertyValue), isNull(propertyName));
		if (Operator.leex == op)
			return or(le(propertyName, propertyValue), isNull(propertyName));
		if (Operator.eqex == op)
			return or(eq(propertyName, propertyValue), isNull(propertyName));
		if (Operator.neqex == op)
			return or(ne(propertyName, propertyValue), isNull(propertyName));
		if (Operator.likeex == op)
			return or(like(propertyName, propertyValue), isNull(propertyName));
		throw new SeafPersistenceException("Unknown Operator: " + op);
	}

}
